@@ -16,6 +16,7 @@
 
 import abc
 import collections
+import enum
 
 from oslo_utils import excutils
 from oslo_utils import netutils
@@ -28,6 +29,74 @@ import tooz
 TOOZ_BACKENDS_NAMESPACE = "tooz.backends"
 
 
+class Characteristics(enum.Enum):
+    """Attempts to describe the characteristic that a driver supports."""
+
+    DISTRIBUTED_ACROSS_THREADS = 'DISTRIBUTED_ACROSS_THREADS'
+    """Coordinator components when used by multiple **threads** work
+       the same as if those components were only used by a single thread."""
+
+    DISTRIBUTED_ACROSS_PROCESSES = 'DISTRIBUTED_ACROSS_PROCESSES'
+    """Coordinator components when used by multiple **processes** work
+       the same as if those components were only used by a single thread."""
+
+    DISTRIBUTED_ACROSS_HOSTS = 'DISTRIBUTED_ACROSS_HOSTS'
+    """Coordinator components when used by multiple **hosts** work
+       the same as if those components were only used by a single thread."""
+
+    LINEARIZABLE = 'LINEARIZABLE'
+    """The driver has the following properties:
+
+    * Ensures each operation must take place before its
+      completion time.
+    * Any operation invoked subsequently must take place
+      after the invocation and by extension, after the original operation
+      itself.
+    """
+
+    SEQUENTIAL = 'SEQUENTIAL'
+    """The driver has the following properties:
+
+    * Operations can take effect before or after completion – but all
+      operations retain the constraint that operations from any given process
+      must take place in that processes order.
+    """
+
+    CAUSAL = 'CAUSAL'
+    """The driver has the following properties:
+
+    * Does **not** have to enforce the order of every
+      operation from a process, perhaps, only causally related operations
+      must occur in order.
+    """
+
+    SERIALIZABLE = 'SERIALIZABLE'
+    """The driver has the following properties:
+
+    * The history of **all** operations is equivalent to
+      one that took place in some single atomic order but with unknown
+      invocation and completion times - it places no bounds on
+      time or order.
+    """
+
+    SAME_VIEW_UNDER_PARTITIONS = 'SAME_VIEW_UNDER_PARTITIONS'
+    """When a client is connected to a server and that server is partitioned
+    from a group of other servers it will (somehow) have the same view of
+    data as a client connected to a server on the other side of the
+    partition (typically this is accomplished by write availability being
+    lost and therefore nothing can change).
+    """
+
+    SAME_VIEW_ACROSS_CLIENTS = 'SAME_VIEW_ACROSS_CLIENTS'
+    """A client connected to one server will *always* have the same view
+    every other client will have (no matter what server those other
+    clients are connected to). Typically this is a sacrifice in
+    write availability because before a write can be acknowledged it must
+    be acknowledged by *all* servers in a cluster (so that all clients
+    that are connected to those servers read the exact *same* thing).
+    """
+
+
 class Hooks(list):
     def run(self, *args, **kwargs):
         return list(map(lambda cb: cb(*args, **kwargs), self))
@@ -72,6 +141,12 @@ class CoordinationDriver(object):
     backing store.
     """
 
+    CHARACTERISTICS = ()
+    """
+    Tuple of :py:class:`~tooz.coordination.Characteristics` introspectable
+    enum member(s) that can be used to interogate how this driver works.
+    """
+
     def __init__(self):
         self._started = False
         self._hooks_join_group = collections.defaultdict(Hooks)
@@ -81,8 +156,8 @@ class CoordinationDriver(object):
         self._group_members = collections.defaultdict(set)
 
     def _has_hooks_for_group(self, group_id):
-        return (len(self._hooks_join_group[group_id])
-                + len(self._hooks_leave_group[group_id]))
+        return (len(self._hooks_join_group[group_id]) +
+                len(self._hooks_leave_group[group_id]))
 
     @staticmethod
     def run_watchers(timeout=None):
@@ -120,8 +195,8 @@ class CoordinationDriver(object):
                          this group
         """
         self._hooks_join_group[group_id].remove(callback)
-        if (not self._has_hooks_for_group(group_id)
-           and group_id in self._group_members):
+        if (not self._has_hooks_for_group(group_id) and
+           group_id in self._group_members):
             del self._group_members[group_id]
 
     @abc.abstractmethod
@@ -147,8 +222,8 @@ class CoordinationDriver(object):
                          this group
         """
         self._hooks_leave_group[group_id].remove(callback)
-        if (not self._has_hooks_for_group(group_id)
-           and group_id in self._group_members):
+        if (not self._has_hooks_for_group(group_id) and
+           group_id in self._group_members):
             del self._group_members[group_id]
 
     @abc.abstractmethod
@@ -279,8 +354,7 @@ class CoordinationDriver(object):
 
     @staticmethod
     def get_members(group_id):
-        """Return the list of all members ids of the specified group
-        asynchronously.
+        """Return the list of all members ids of the specified group.
 
         :returns: list of all created group ids
         :rtype: CoordAsyncResult
@@ -315,8 +389,7 @@ class CoordinationDriver(object):
 
     @staticmethod
     def update_capabilities(group_id, capabilities):
-        """Update capabilities of the caller in the specified group
-        asynchronously.
+        """Update member capabilities in the specified group.
 
         :param group_id: the id of the group of the current member
         :type group_id: str
@@ -352,9 +425,10 @@ class CoordinationDriver(object):
 
     @staticmethod
     def heartbeat():
-        """Method to run once in a while to be sure that the member is not dead
-        and is still an active member of a group.
+        """Update member status to indicate it is still alive.
 
+        Method to run once in a while to be sure that the member is not dead
+        and is still an active member of a group.
 
         """
         pass
@@ -362,14 +436,17 @@ class CoordinationDriver(object):
 
 @six.add_metaclass(abc.ABCMeta)
 class CoordAsyncResult(object):
-    """Representation of an asynchronous task, every call API
-    returns an CoordAsyncResult object on which the result or
+    """Representation of an asynchronous task.
+
+    Every call API returns an CoordAsyncResult object on which the result or
     the status of the task can be requested.
+
     """
 
     @abc.abstractmethod
     def get(self, timeout=10):
         """Retrieve the result of the corresponding asynchronous call.
+
         :param timeout: block until the timeout expire.
         :type timeout: float
         """
@@ -412,13 +489,19 @@ class _RunWatchersMixin(object):
             return result
 
 
-def get_coordinator(backend_url, member_id, **kwargs):
+def get_coordinator(backend_url, member_id,
+                    characteristics=frozenset(), **kwargs):
     """Initialize and load the backend.
 
     :param backend_url: the backend URL to use
     :type backend: str
     :param member_id: the id of the member
     :type member_id: str
+    :param characteristics: set
+    :type characteristics: set of :py:class:`.Characteristics` that will
+                           be matched to the requested driver (this **will**
+                           become a **required** parameter in a future tooz
+                           version)
     :param kwargs: additional coordinator options (these take precedence over
                    options of the **same** name found in the ``backend_url``
                    arguments query string)
@@ -434,21 +517,35 @@ def get_coordinator(backend_url, member_id, **kwargs):
                 options[k] = v
     else:
         options = parsed_qs
-    return driver.DriverManager(
+    d = driver.DriverManager(
         namespace=TOOZ_BACKENDS_NAMESPACE,
         name=parsed_url.scheme,
         invoke_on_load=True,
         invoke_args=(member_id, parsed_url, options)).driver
+    characteristics = set(characteristics)
+    driver_characteristics = set(getattr(d, 'CHARACTERISTICS', set()))
+    missing_characteristics = characteristics - driver_characteristics
+    if missing_characteristics:
+        raise ToozDriverChosenPoorly("Desired characteristics %s"
+                                     " is not a strict subset of driver"
+                                     " characteristics %s, %s"
+                                     " characteristics were not found"
+                                     % (characteristics,
+                                        driver_characteristics,
+                                        missing_characteristics))
+    return d
 
 
 class ToozError(Exception):
-    """Exception raised when an internal error occurs, for instance in
-    case of server internal error.
+    """Exception raised when an internal error occurs.
+
+    Raised for instance in case of server internal error.
 
     :ivar cause: the cause of the exception being raised, when not none this
                  will itself be an exception instance, this is useful for
                  creating a chain of exceptions for versions of python where
                  this is not yet implemented/supported natively.
+
     """
 
     def __init__(self, message, cause=None):
@@ -456,10 +553,12 @@ class ToozError(Exception):
         self.cause = cause
 
 
+class ToozDriverChosenPoorly(ToozError):
+    """Raised when a driver does not match desired characteristics."""
+
+
 class ToozConnectionError(ToozError):
-    """Exception raised when the client cannot manage to connect to the
-    server.
-    """
+    """Exception raised when the client cannot connect to the server."""
 
 
 class OperationTimedOut(ToozError):
@@ -471,9 +570,7 @@ class LockAcquireFailed(ToozError):
 
 
 class GroupNotCreated(ToozError):
-    """Exception raised when the caller request a group which does
-    not exist.
-    """
+    """Exception raised when the caller request an nonexistent group."""
     def __init__(self, group_id):
         self.group_id = group_id
         super(GroupNotCreated, self).__init__(
@@ -481,9 +578,7 @@ class GroupNotCreated(ToozError):
 
 
 class GroupAlreadyExist(ToozError):
-    """Exception raised when the caller try to create a group which already
-    exist.
-    """
+    """Exception raised trying to create an already existing group."""
     def __init__(self, group_id):
         self.group_id = group_id
         super(GroupAlreadyExist, self).__init__(
@@ -491,9 +586,7 @@ class GroupAlreadyExist(ToozError):
 
 
 class MemberAlreadyExist(ToozError):
-    """Exception raised when the caller try to join a group but a member
-    with the same identifier belongs to that group.
-    """
+    """Exception raised trying to join a group already joined."""
     def __init__(self, group_id, member_id):
         self.group_id = group_id
         self.member_id = member_id
@@ -503,9 +596,7 @@ class MemberAlreadyExist(ToozError):
 
 
 class MemberNotJoined(ToozError):
-    """Exception raised when the caller try to access a member which does not
-    belongs to the specified group.
-    """
+    """Exception raised trying to access a member not in a group."""
     def __init__(self, group_id, member_id):
         self.group_id = group_id
         self.member_id = member_id
